@@ -1,4 +1,3 @@
-# -*- encoding: utf-8 -*-
 # Copyright 2011 OpenStack Foundation.
 # All Rights Reserved.
 #
@@ -32,26 +31,50 @@ def generate_id(name):
 class WarningsFixture(fixtures.Fixture):
 
     def __init__(self, action="always", category=DeprecationWarning):
-        super(WarningsFixture, self).__init__()
+        super().__init__()
         self.action = action
         self.category = category
 
     def setUp(self):
-        super(WarningsFixture, self).setUp()
+        super().setUp()
         self._w = warnings.catch_warnings(record=True)
         self.log = self._w.__enter__()
         self.addCleanup(self._w.__exit__)
         warnings.simplefilter(self.action, self.category)
 
+    def __len__(self):
+        return len(self.log)
 
-class Object(object):
+    def __getitem__(self, item):
+        return self.log[item]
+
+
+class Object:
     pass
 
 
+class TestContext(context.RequestContext):
+    """A test context with additional members
+
+    This is representative of how at least some of our consumers use the
+    RequestContext class in their projects.
+    """
+    FROM_DICT_EXTRA_KEYS = ['auth_token_info']
+
+    def __init__(self, auth_token_info=None, **kwargs):
+        super().__init__(**kwargs)
+        self.auth_token_info = auth_token_info
+
+    def to_dict(self):
+        d = super().to_dict()
+        d['auth_token_info'] = self.auth_token_info
+        return d
+
+
 class ContextTest(test_base.BaseTestCase):
 
     def setUp(self):
-        super(ContextTest, self).setUp()
+        super().setUp()
         self.warnings = self.useFixture(WarningsFixture())
         self.useFixture(fixture.ClearRequestContext())
 
@@ -93,7 +116,7 @@ class ContextTest(test_base.BaseTestCase):
         self.assertIsInstance(ctx, context.RequestContext)
         self.assertTrue(ctx.is_admin)
         self.assertFalse(ctx.show_deleted)
-        self.assertIsNone(ctx.tenant)
+        self.assertIsNone(ctx.project_id)
 
     def test_admin_context_show_deleted_flag_set(self):
         ctx = context.get_admin_context(show_deleted=True)
@@ -105,7 +128,7 @@ class ContextTest(test_base.BaseTestCase):
             "auth_token": "token1",
             "user": "user1",
             "user_name": "user1_name",
-            "tenant": "tenant1",
+            "project_id": "tenant1",
             "project_name": "tenant1_name",
             "domain": "domain1",
             "domain_name": "domain1_name",
@@ -117,26 +140,30 @@ class ContextTest(test_base.BaseTestCase):
             "read_only": True,
             "show_deleted": True,
             "request_id": "request1",
+            "global_request_id": "req-uuid",
             "resource_uuid": "instance1",
-            "extra_data": "foo"
+            "extra_data": "foo",
+            "system_scope": "all"
         }
         ctx = context.RequestContext.from_dict(dct)
         self.assertEqual(dct['auth_token'], ctx.auth_token)
-        self.assertEqual(dct['user'], ctx.user)
-        self.assertEqual(dct['tenant'], ctx.tenant)
-        self.assertEqual(dct['domain'], ctx.domain)
-        self.assertEqual(dct['user_domain'], ctx.user_domain)
-        self.assertEqual(dct['project_domain'], ctx.project_domain)
+        self.assertEqual(dct['user'], ctx.user_id)
+        self.assertEqual(dct['project_id'], ctx.project_id)
+        self.assertEqual(dct['domain'], ctx.domain_id)
+        self.assertEqual(dct['user_domain'], ctx.user_domain_id)
+        self.assertEqual(dct['project_domain'], ctx.project_domain_id)
         self.assertTrue(ctx.is_admin)
         self.assertTrue(ctx.read_only)
         self.assertTrue(ctx.show_deleted)
         self.assertEqual(dct['request_id'], ctx.request_id)
+        self.assertEqual(dct['global_request_id'], ctx.global_request_id)
         self.assertEqual(dct['resource_uuid'], ctx.resource_uuid)
         self.assertEqual(dct['user_name'], ctx.user_name)
         self.assertEqual(dct['project_name'], ctx.project_name)
         self.assertEqual(dct['domain_name'], ctx.domain_name)
         self.assertEqual(dct['user_domain_name'], ctx.user_domain_name)
         self.assertEqual(dct['project_domain_name'], ctx.project_domain_name)
+        self.assertEqual(dct['system_scope'], ctx.system_scope)
 
     def test_from_dict_unknown_keys(self):
         dct = {
@@ -149,8 +176,8 @@ class ContextTest(test_base.BaseTestCase):
         }
         ctx = context.RequestContext.from_dict(dct)
         self.assertEqual("token1", ctx.auth_token)
-        self.assertEqual("user1", ctx.user)
-        self.assertIsNone(ctx.tenant)
+        self.assertEqual("user1", ctx.user_id)
+        self.assertIsNone(ctx.project_id)
         self.assertFalse(ctx.is_admin)
         self.assertTrue(ctx.read_only)
         self.assertRaises(KeyError, lambda: ctx.__dict__['color'])
@@ -170,10 +197,17 @@ class ContextTest(test_base.BaseTestCase):
         self.assertEqual("token1", ctx.auth_token)
         self.assertEqual("user2", ctx.user)
         self.assertEqual("project1", ctx.project_name)
-        self.assertIsNone(ctx.tenant)
+        self.assertIsNone(ctx.project_id)
         self.assertFalse(ctx.is_admin)
         self.assertTrue(ctx.read_only)
 
+    def test_from_dict_extended(self):
+        initial = TestContext(auth_token_info='foo')
+        dct = initial.to_dict()
+        final = TestContext.from_dict(dct)
+        self.assertEqual('foo', final.auth_token_info)
+        self.assertEqual(dct, final.to_dict())
+
     def test_is_user_context(self):
         self.assertFalse(context.is_user_context(None))
         ctx = context.RequestContext(is_admin=True)
@@ -188,38 +222,80 @@ class ContextTest(test_base.BaseTestCase):
         user_id = generate_id(user_name)
         project_name = uuid.uuid4().hex
         project_id = generate_id(project_name)
+        domain_name = uuid.uuid4().hex
+        domain_id = generate_id(domain_name)
         user_domain_name = uuid.uuid4().hex
         user_domain_id = generate_id(user_domain_name)
         project_domain_name = uuid.uuid4().hex
         project_domain_id = generate_id(project_domain_name)
         roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
         request_id = uuid.uuid4().hex
+        global_request_id = uuid.uuid4().hex
+        service_token = uuid.uuid4().hex
+        service_user_id = uuid.uuid4().hex
+        service_user_name = uuid.uuid4().hex
+        service_user_domain_id = uuid.uuid4().hex
+        service_user_domain_name = uuid.uuid4().hex
+        service_project_id = uuid.uuid4().hex
+        service_project_name = uuid.uuid4().hex
+        service_project_domain_id = uuid.uuid4().hex
+        service_project_domain_name = uuid.uuid4().hex
+        service_roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
 
-        environ = {'HTTP_X_AUTH_TOKEN': auth_token,
-                   'HTTP_X_USER_ID': user_id,
-                   'HTTP_X_PROJECT_ID': project_id,
-                   'HTTP_X_USER_DOMAIN_ID': user_domain_id,
-                   'HTTP_X_PROJECT_DOMAIN_ID': project_domain_id,
-                   'HTTP_X_ROLES': ','.join(roles),
-                   'HTTP_X_USER_NAME': user_name,
-                   'HTTP_X_PROJECT_NAME': project_name,
-                   'HTTP_X_USER_DOMAIN_NAME': user_domain_name,
-                   'HTTP_X_PROJECT_DOMAIN_NAME': project_domain_name,
-                   'openstack.request_id': request_id}
+        environ = {
+            'HTTP_X_AUTH_TOKEN': auth_token,
+            'HTTP_X_USER_ID': user_id,
+            'HTTP_X_PROJECT_ID': project_id,
+            'HTTP_X_DOMAIN_ID': domain_id,
+            'HTTP_X_USER_DOMAIN_ID': user_domain_id,
+            'HTTP_X_PROJECT_DOMAIN_ID': project_domain_id,
+            'HTTP_X_ROLES': ','.join(roles),
+            'HTTP_X_USER_NAME': user_name,
+            'HTTP_X_PROJECT_NAME': project_name,
+            'HTTP_X_USER_DOMAIN_NAME': user_domain_name,
+            'HTTP_X_PROJECT_DOMAIN_NAME': project_domain_name,
+            'HTTP_X_SERVICE_TOKEN': service_token,
+            'HTTP_X_SERVICE_USER_ID': service_user_id,
+            'HTTP_X_SERVICE_USER_NAME': service_user_name,
+            'HTTP_X_SERVICE_USER_DOMAIN_ID': service_user_domain_id,
+            'HTTP_X_SERVICE_USER_DOMAIN_NAME': service_user_domain_name,
+            'HTTP_X_SERVICE_PROJECT_ID': service_project_id,
+            'HTTP_X_SERVICE_PROJECT_NAME': service_project_name,
+            'HTTP_X_SERVICE_PROJECT_DOMAIN_ID': service_project_domain_id,
+            'HTTP_X_SERVICE_PROJECT_DOMAIN_NAME': service_project_domain_name,
+            'HTTP_X_SERVICE_ROLES': ','.join(service_roles),
+            'openstack.request_id': request_id,
+            'openstack.global_request_id': global_request_id,
+        }
 
         ctx = context.RequestContext.from_environ(environ)
 
         self.assertEqual(auth_token, ctx.auth_token)
-        self.assertEqual(user_id, ctx.user)
+        self.assertEqual(user_id, ctx.user_id)
         self.assertEqual(user_name, ctx.user_name)
-        self.assertEqual(project_id, ctx.tenant)
+        self.assertEqual(project_id, ctx.project_id)
+        self.assertEqual(domain_id, ctx.domain_id)
         self.assertEqual(project_name, ctx.project_name)
-        self.assertEqual(user_domain_id, ctx.user_domain)
+        self.assertEqual(user_domain_id, ctx.user_domain_id)
         self.assertEqual(user_domain_name, ctx.user_domain_name)
-        self.assertEqual(project_domain_id, ctx.project_domain)
+        self.assertEqual(project_domain_id, ctx.project_domain_id)
         self.assertEqual(project_domain_name, ctx.project_domain_name)
         self.assertEqual(roles, ctx.roles)
         self.assertEqual(request_id, ctx.request_id)
+        self.assertEqual(global_request_id, ctx.global_request_id)
+        self.assertEqual(service_token, ctx.service_token)
+        self.assertEqual(service_user_id, ctx.service_user_id)
+        self.assertEqual(service_user_name, ctx.service_user_name)
+        self.assertEqual(service_user_domain_id, ctx.service_user_domain_id)
+        self.assertEqual(service_user_domain_name,
+                         ctx.service_user_domain_name)
+        self.assertEqual(service_project_id, ctx.service_project_id)
+        self.assertEqual(service_project_name, ctx.service_project_name)
+        self.assertEqual(service_project_domain_id,
+                         ctx.service_project_domain_id)
+        self.assertEqual(service_project_domain_name,
+                         ctx.service_project_domain_name)
+        self.assertEqual(service_roles, ctx.service_roles)
 
     def test_from_environ_no_roles(self):
         ctx = context.RequestContext.from_environ(environ={})
@@ -237,7 +313,7 @@ class ContextTest(test_base.BaseTestCase):
 
         environ = {'HTTP_X_TENANT_ID': value}
         ctx = context.RequestContext.from_environ(environ=environ)
-        self.assertEqual(value, ctx.tenant)
+        self.assertEqual(value, ctx.project_id)
 
         environ = {'HTTP_X_STORAGE_TOKEN': value}
         ctx = context.RequestContext.from_environ(environ=environ)
@@ -245,7 +321,7 @@ class ContextTest(test_base.BaseTestCase):
 
         environ = {'HTTP_X_TENANT': value}
         ctx = context.RequestContext.from_environ(environ=environ)
-        self.assertEqual(value, ctx.tenant)
+        self.assertEqual(value, ctx.project_id)
 
         environ = {'HTTP_X_ROLE': value}
         ctx = context.RequestContext.from_environ(environ=environ)
@@ -274,11 +350,11 @@ class ContextTest(test_base.BaseTestCase):
                    'HTTP_X_PROJECT_ID': new}
 
         ctx = context.RequestContext.from_environ(environ=environ)
-        self.assertEqual(new, ctx.tenant)
+        self.assertEqual(new, ctx.project_id)
 
         ctx = context.RequestContext.from_environ(environ=environ,
-                                                  tenant=override)
-        self.assertEqual(override, ctx.tenant)
+                                                  project_id=override)
+        self.assertEqual(override, ctx.project_id)
 
         environ = {'HTTP_X_TENANT_NAME': old,
                    'HTTP_X_PROJECT_NAME': new}
@@ -287,9 +363,11 @@ class ContextTest(test_base.BaseTestCase):
         self.assertEqual(new, ctx.project_name)
 
     def test_from_environ_strip_roles(self):
-        environ = {'HTTP_X_ROLES': ' abc\t,\ndef\n,ghi\n\n'}
+        environ = {'HTTP_X_ROLES': ' abc\t,\ndef\n,ghi\n\n',
+                   'HTTP_X_SERVICE_ROLES': ' jkl\t,\nmno\n,pqr\n\n'}
         ctx = context.RequestContext.from_environ(environ=environ)
         self.assertEqual(['abc', 'def', 'ghi'], ctx.roles)
+        self.assertEqual(['jkl', 'mno', 'pqr'], ctx.service_roles)
 
     def test_environ_admin_project(self):
         environ = {}
@@ -325,7 +403,7 @@ class ContextTest(test_base.BaseTestCase):
     def test_values(self):
         auth_token = "token1"
         # test unicode support
-        user_name = u"John Gāo"
+        user_name = "John Gāo"
         user_id = generate_id(user_name)
         project_name = 'tenant1'
         project_id = generate_id(project_name)
@@ -339,12 +417,13 @@ class ContextTest(test_base.BaseTestCase):
         read_only = True
         show_deleted = True
         request_id = "id1"
+        global_request_id = "req-id1"
         resource_uuid = "uuid1"
 
         ctx = context.RequestContext(auth_token=auth_token,
                                      user=user_id,
                                      user_name=user_name,
-                                     tenant=project_id,
+                                     project_id=project_id,
                                      project_name=project_name,
                                      domain=domain_id,
                                      domain_name=domain_name,
@@ -356,17 +435,18 @@ class ContextTest(test_base.BaseTestCase):
                                      read_only=read_only,
                                      show_deleted=show_deleted,
                                      request_id=request_id,
+                                     global_request_id=global_request_id,
                                      resource_uuid=resource_uuid)
         self.assertEqual(auth_token, ctx.auth_token)
-        self.assertEqual(user_id, ctx.user)
+        self.assertEqual(user_id, ctx.user_id)
         self.assertEqual(user_name, ctx.user_name)
-        self.assertEqual(project_id, ctx.tenant)
+        self.assertEqual(project_id, ctx.project_id)
         self.assertEqual(project_name, ctx.project_name)
-        self.assertEqual(domain_id, ctx.domain)
+        self.assertEqual(domain_id, ctx.domain_id)
         self.assertEqual(domain_name, ctx.domain_name)
-        self.assertEqual(user_domain_id, ctx.user_domain)
+        self.assertEqual(user_domain_id, ctx.user_domain_id)
         self.assertEqual(user_domain_name, ctx.user_domain_name)
-        self.assertEqual(project_domain_id, ctx.project_domain)
+        self.assertEqual(project_domain_id, ctx.project_domain_id)
         self.assertEqual(project_domain_name, ctx.project_domain_name)
         self.assertEqual(is_admin, ctx.is_admin)
         self.assertEqual(read_only, ctx.read_only)
@@ -377,7 +457,7 @@ class ContextTest(test_base.BaseTestCase):
         d = ctx.to_dict()
         self.assertIn('auth_token', d)
         self.assertIn('user', d)
-        self.assertIn('tenant', d)
+        self.assertIn('project_id', d)
         self.assertIn('domain', d)
         self.assertIn('user_domain', d)
         self.assertIn('project_domain', d)
@@ -395,7 +475,7 @@ class ContextTest(test_base.BaseTestCase):
         self.assertNotIn('project_domain_name', d)
 
         self.assertEqual(auth_token, d['auth_token'])
-        self.assertEqual(project_id, d['tenant'])
+        self.assertEqual(project_id, d['project_id'])
         self.assertEqual(domain_id, d['domain'])
         self.assertEqual(user_domain_id, d['user_domain'])
         self.assertEqual(project_domain_id, d['project_domain'])
@@ -404,15 +484,16 @@ class ContextTest(test_base.BaseTestCase):
         self.assertEqual(show_deleted, d['show_deleted'])
         self.assertEqual(request_id, d['request_id'])
         self.assertEqual(resource_uuid, d['resource_uuid'])
-        user_identity = "%s %s %s %s %s" % (user_id, project_id, domain_id,
-                                            user_domain_id, project_domain_id)
+        user_identity = "{} {} {} {} {}".format(
+            user_id, project_id, domain_id, user_domain_id, project_domain_id)
         self.assertEqual(user_identity, d['user_identity'])
         self.assertEqual([], d['roles'])
 
         d = ctx.get_logging_values()
         self.assertIn('auth_token', d)
+        self.assertEqual(d['auth_token'], '***')
         self.assertIn('user', d)
-        self.assertIn('tenant', d)
+        self.assertIn('project_id', d)
         self.assertIn('domain', d)
         self.assertIn('user_domain', d)
         self.assertIn('project_domain', d)
@@ -420,6 +501,7 @@ class ContextTest(test_base.BaseTestCase):
         self.assertIn('read_only', d)
         self.assertIn('show_deleted', d)
         self.assertIn('request_id', d)
+        self.assertIn('global_request_id', d)
         self.assertIn('resource_uuid', d)
         self.assertIn('user_identity', d)
         self.assertIn('roles', d)
@@ -435,6 +517,25 @@ class ContextTest(test_base.BaseTestCase):
         self.assertEqual(user_domain_name, d['user_domain_name'])
         self.assertEqual(project_domain_name, d['project_domain_name'])
 
+    def test_auth_token_info_removed_logging_values(self):
+        ctx = TestContext(auth_token_info={'auth_token': 'topsecret'})
+        d = ctx.get_logging_values()
+        self.assertNotIn('auth_token_info', d)
+
+    def test_auth_token_info_removed_redacted_context(self):
+        userid = 'foo'
+        ctx = TestContext(
+            auth_token_info={'auth_token': 'topsecret'},
+            service_token="1234567",
+            auth_token="8901234",
+            user_id=userid)
+        safe_ctxt = ctx.redacted_copy()
+        self.assertIsNone(safe_ctxt.auth_token_info)
+        self.assertIsNone(safe_ctxt.service_token)
+        self.assertIsNone(safe_ctxt.auth_token)
+        self.assertEqual(userid, safe_ctxt.user_id)
+        self.assertNotEqual(ctx, safe_ctxt)
+
     def test_dict_empty_user_identity(self):
         ctx = context.RequestContext()
         d = ctx.to_dict()
@@ -449,47 +550,179 @@ class ContextTest(test_base.BaseTestCase):
         id2 = context.generate_request_id()
         self.assertNotEqual(id1, id2)
 
+    def test_no_global_id_by_default(self):
+        ctx = context.RequestContext()
+        self.assertIsNone(ctx.global_request_id)
+        d = ctx.to_dict()
+        self.assertIsNone(d['global_request_id'])
+
     def test_policy_dict(self):
         user = uuid.uuid4().hex
         user_domain = uuid.uuid4().hex
-        tenant = uuid.uuid4().hex
+        project_id = uuid.uuid4().hex
         project_domain = uuid.uuid4().hex
         roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
+        service_user_id = uuid.uuid4().hex
+        service_project_id = uuid.uuid4().hex
+        service_roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
 
         # default is_admin_project is True
         ctx = context.RequestContext(user=user,
                                      user_domain=user_domain,
-                                     tenant=tenant,
+                                     project_id=project_id,
+                                     project_domain=project_domain,
+                                     roles=roles,
+                                     service_user_id=service_user_id,
+                                     service_project_id=service_project_id,
+                                     service_roles=service_roles)
+
+        self.assertEqual({'user_id': user,
+                          'user_domain_id': user_domain,
+                          'system_scope': None,
+                          'domain_id': None,
+                          'project_id': project_id,
+                          'project_domain_id': project_domain,
+                          'roles': roles,
+                          'is_admin_project': True,
+                          'service_user_id': service_user_id,
+                          'service_user_domain_id': None,
+                          'service_project_id': service_project_id,
+                          'service_project_domain_id': None,
+                          'service_roles': service_roles},
+                         ctx.to_policy_values())
+
+        # NOTE(lbragstad): This string has special meaning in that the value
+        # ``all`` represents the entire deployment system.
+        system_all = 'all'
+
+        ctx = context.RequestContext(user=user,
+                                     user_domain=user_domain,
+                                     system_scope=system_all,
+                                     roles=roles,
+                                     service_user_id=service_user_id,
+                                     service_project_id=service_project_id,
+                                     service_roles=service_roles)
+
+        self.assertEqual({'user_id': user,
+                          'user_domain_id': user_domain,
+                          'system_scope': system_all,
+                          'domain_id': None,
+                          'project_id': None,
+                          'project_domain_id': None,
+                          'roles': roles,
+                          'is_admin_project': True,
+                          'service_user_id': service_user_id,
+                          'service_user_domain_id': None,
+                          'service_project_id': service_project_id,
+                          'service_project_domain_id': None,
+                          'service_roles': service_roles},
+                         ctx.to_policy_values())
+
+        # context representing a domain-scoped token.
+        domain_id = uuid.uuid4().hex
+        ctx = context.RequestContext(user=user,
+                                     user_domain=user_domain,
+                                     domain_id=domain_id,
+                                     roles=roles,
+                                     service_user_id=service_user_id,
+                                     service_project_id=service_project_id,
+                                     service_roles=service_roles)
+
+        self.assertEqual({'user_id': user,
+                          'user_domain_id': user_domain,
+                          'system_scope': None,
+                          'domain_id': domain_id,
+                          'project_id': None,
+                          'project_domain_id': None,
+                          'roles': roles,
+                          'is_admin_project': True,
+                          'service_user_id': service_user_id,
+                          'service_user_domain_id': None,
+                          'service_project_id': service_project_id,
+                          'service_project_domain_id': None,
+                          'service_roles': service_roles},
+                         ctx.to_policy_values())
+
+        ctx = context.RequestContext(user=user,
+                                     user_domain=user_domain,
+                                     project_id=project_id,
+                                     project_domain=project_domain,
+                                     roles=roles,
+                                     is_admin_project=False,
+                                     service_user_id=service_user_id,
+                                     service_project_id=service_project_id,
+                                     service_roles=service_roles)
+
+        self.assertEqual({'user_id': user,
+                          'user_domain_id': user_domain,
+                          'system_scope': None,
+                          'domain_id': None,
+                          'project_id': project_id,
+                          'project_domain_id': project_domain,
+                          'roles': roles,
+                          'is_admin_project': False,
+                          'service_user_id': service_user_id,
+                          'service_user_domain_id': None,
+                          'service_project_id': service_project_id,
+                          'service_project_domain_id': None,
+                          'service_roles': service_roles},
+                         ctx.to_policy_values())
+
+    def test_policy_deprecations(self):
+        user = uuid.uuid4().hex
+        user_domain = uuid.uuid4().hex
+        project_id = uuid.uuid4().hex
+        project_domain = uuid.uuid4().hex
+        roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
+
+        ctx = context.RequestContext(user=user,
+                                     user_domain=user_domain,
+                                     project_id=project_id,
                                      project_domain=project_domain,
                                      roles=roles)
 
-        self.assertEqual({'user_id': user,
-                          'user_domain_id': user_domain,
-                          'project_id': tenant,
-                          'project_domain_id': project_domain,
-                          'roles': roles,
-                          'is_admin_project': True},
-                         ctx.to_policy_values())
+        policy = ctx.to_policy_values()
+        key = uuid.uuid4().hex
+        val = uuid.uuid4().hex
 
-        # is_admin_project False gets passed through
-        ctx = context.RequestContext(user=user,
-                                     user_domain=user_domain,
-                                     tenant=tenant,
-                                     project_domain=project_domain,
-                                     roles=roles,
-                                     is_admin_project=False)
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
 
-        self.assertEqual({'user_id': user,
-                          'user_domain_id': user_domain,
-                          'project_id': tenant,
-                          'project_domain_id': project_domain,
-                          'roles': roles,
-                          'is_admin_project': False},
-                         ctx.to_policy_values())
+            # no warning triggered by adding key to dict
+            policy[key] = val
+            self.assertEqual(0, len(w))
 
-    def test_positional_args(self):
-        context.RequestContext('abc', 'def')
+            # warning triggered by fetching key from dict
+            self.assertIs(val, policy[key])
+            self.assertEqual(1, len(w))
+            self.assertIn(key, str(w[0].message))
 
-        self.assertEqual(1, len(self.warnings.log))
-        self.assertIn('__init__ takes at most 1 positional',
-                      str(self.warnings.log[0].message))
+    def test_deprecated_args(self):
+        user_id = uuid.uuid4().hex
+        project_id = uuid.uuid4().hex
+        domain_id = uuid.uuid4().hex
+        user_domain_id = uuid.uuid4().hex
+        project_domain_id = uuid.uuid4().hex
+
+        ctx = context.RequestContext(user_id=user_id,
+                                     project_id=project_id,
+                                     domain_id=domain_id,
+                                     user_domain_id=user_domain_id,
+                                     project_domain_id=project_domain_id)
+
+        self.assertEqual(0, len(self.warnings))
+        self.assertEqual(user_id, ctx.user_id)
+        self.assertEqual(project_id, ctx.project_id)
+        self.assertEqual(domain_id, ctx.domain_id)
+        self.assertEqual(user_domain_id, ctx.user_domain_id)
+        self.assertEqual(project_domain_id, ctx.project_domain_id)
+
+        self.assertEqual(0, len(self.warnings))
+        self.assertEqual(user_id, ctx.user)
+        self.assertEqual(1, len(self.warnings))
+        self.assertEqual(domain_id, ctx.domain)
+        self.assertEqual(2, len(self.warnings))
+        self.assertEqual(user_domain_id, ctx.user_domain)
+        self.assertEqual(3, len(self.warnings))
+        self.assertEqual(project_domain_id, ctx.project_domain)
+        self.assertEqual(4, len(self.warnings))
